@@ -36,12 +36,20 @@
 #include <ucred.h>
 #endif
 
-#include "gnome-keyring.h"
-#include "gnome-keyring-private.h"
-#include "gnome-keyring-proto.h"
 #include "gnome-keyring-daemon.h"
 #include "mkdtemp.h"
 
+#include "common/gkr-buffer.h"
+
+#include "keyrings/gkr-keyrings.h"
+
+#include "library/gnome-keyring.h"
+#include "library/gnome-keyring-memory.h"
+#include "library/gnome-keyring-private.h"
+#include "library/gnome-keyring-proto.h"
+
+#include "ui/gkr-ask-daemon.h"
+
 #ifndef HAVE_SOCKLEN_T
 #define socklen_t int
 #endif
@@ -66,21 +74,23 @@ typedef struct {
 	
 	guint input_watch;
 	GIOChannel *input_channel;
-	GString *input_buffer;
+	GkrBuffer input_buffer;
 	gint input_pos;
 
-	gpointer ask;
+	GkrAskRequest* ask;
 
-	GList *access_requests;
+	GList *ask_requests;
+	GList *granted_requests;
+	gpointer collect_data;
 
 	guint output_watch;
-	GString *output_buffer;
+	GkrBuffer output_buffer;
 	gint output_pos;
 } GnomeKeyringClient;
 
-char tmp_dir[1024];
-char socket_path[1024];
-GList *clients = NULL;
+static char tmp_dir[1024] = { 0, };
+static char socket_path[1024] = { 0, };
+static GList *clients = NULL;
 
 #if 0
 #define debug_print(x) g_print x
@@ -90,7 +100,7 @@ GList *clients = NULL;
 
 
 static void gnome_keyring_client_state_machine (GnomeKeyringClient *client);
-
+static void ask_result (GkrAskRequest *ask, gpointer data);
 
 static gboolean
 set_local_creds (int fd, gboolean on)
@@ -229,12 +239,8 @@ gnome_keyring_client_free (GnomeKeyringClient *client)
 	clients = g_list_remove (clients, client);
 
 
-	if (client->input_buffer != NULL) {
-		g_string_free (client->input_buffer, TRUE);
-	}
-	if (client->output_buffer != NULL) {
-		g_string_free (client->output_buffer, TRUE);
-	}
+	gkr_buffer_uninit (&client->input_buffer);
+	gkr_buffer_uninit (&client->output_buffer);
 
 	if (client->app_ref != NULL) {
 		gnome_keyring_application_ref_free (client->app_ref);
@@ -253,7 +259,8 @@ gnome_keyring_client_free (GnomeKeyringClient *client)
 	}
 
 	if (client->ask != NULL) {
-		gnome_keyring_cancel_ask (client->ask);
+		g_signal_handlers_disconnect_by_func (client->ask, ask_result, client);
+		gkr_ask_daemon_cancel (client->ask);
 	}
 	
 	close (client->sock);
@@ -270,12 +277,14 @@ read_packet_with_size (GnomeKeyringClient *client)
 	fd = client->sock;
 	
 	if (client->input_pos < 4) {
-		g_string_set_size (client->input_buffer, 4);
-		res = read (fd, client->input_buffer->str + client->input_pos,
+		gkr_buffer_resize (&client->input_buffer, 4);
+		res = read (fd, client->input_buffer.buf + client->input_pos,
 			    4 - client->input_pos);
 		if (res <= 0) {
-			if (errno != EAGAIN &&
-			    errno != EINTR) {
+			if (errno != EAGAIN && errno != EINTR) {
+				if (res < 0)
+					g_warning ("couldn't read %u bytes from client: %s", 
+					           4, g_strerror (errno));
 				gnome_keyring_client_free (client);
 			}
 			return FALSE;
@@ -285,7 +294,7 @@ read_packet_with_size (GnomeKeyringClient *client)
 	}
 
 	if (client->input_pos >= 4) {
-		if (!gnome_keyring_proto_decode_packet_size (client->input_buffer, &packet_size)) {
+		if (!gnome_keyring_proto_decode_packet_size (&client->input_buffer, &packet_size)) {
 			gnome_keyring_client_free (client);
 			return FALSE;
 		}
@@ -295,13 +304,15 @@ read_packet_with_size (GnomeKeyringClient *client)
 		}
 		
 		g_assert (client->input_pos < packet_size);
-		g_string_set_size (client->input_buffer, packet_size);
+		gkr_buffer_resize (&client->input_buffer, packet_size);
 
-		res = read (fd, client->input_buffer->str + client->input_pos,
+		res = read (fd, client->input_buffer.buf + client->input_pos,
 			    packet_size - client->input_pos);
 		if (res <= 0) {
-			if (errno != EAGAIN &&
-			    errno != EINTR) {
+			if (errno != EAGAIN && errno != EINTR) {
+				if (res < 0)
+					g_warning ("couldn't read %u bytes from client: %s", 
+					           packet_size - client->input_pos, g_strerror (errno));
 				gnome_keyring_client_free (client);
 			}
 			return FALSE;
@@ -317,43 +328,36 @@ read_packet_with_size (GnomeKeyringClient *client)
 	return FALSE;
 }
 
-
-void
-gnome_keyring_client_fixup_for_deleted (GnomeKeyring *keyring, GnomeKeyringItem *item)
+static void 
+ask_list_free (GList *asks)
 {
-	GList *l, *reql;
-	GnomeKeyringClient *client;
-	GnomeKeyringAccessRequest *request;
-
-	for (l = clients; l != NULL; l = l->next) {
-		client = l->data;
-
-		reql = client->access_requests;
-		while (reql != NULL) {
-			request = reql->data;
-			reql = reql->next;
-
-			if ((keyring != NULL && request->keyring == keyring) ||
-			    (item != NULL && request->item == item)) {
-				client->access_requests = g_list_remove (client->access_requests,
-									 request);
-				gnome_keyring_access_request_free (request);
-			}
-		}
-	}
+	GList *l;
+	
+	for (l = asks; l; l = g_list_next (l))
+		g_object_unref (l->data);
+	g_list_free (asks);
 }
 
 static void
-ask_result (GList *access_requests,
-	    gpointer data)
+ask_result (GkrAskRequest *ask, gpointer data)
 {
 	GnomeKeyringClient *client;
 
 	client = data;
+	
+	/* Should match up with the first one in the ask list */
+	g_assert (client->ask == ask);
+	
+	/* Move it to the granted list? */
+	if (ask->response >= GKR_ASK_RESPONSE_ALLOW) {
+		client->granted_requests = g_list_append (client->granted_requests, ask);
+		g_object_ref (ask);
+	}
+	
+	/* And discard it */
+	g_object_unref (client->ask);
 	client->ask = NULL;
 
-	client->access_requests = gnome_keyring_access_request_list_copy (access_requests);
-
 	gnome_keyring_client_state_machine (client);
 }
 
@@ -375,8 +379,7 @@ gnome_keyring_client_state_machine (GnomeKeyringClient *client)
 {
 	GnomeKeyringOpCode op;
 	GIOChannel *channel;
-	GList *access_requests;
-	gpointer ask;
+	GkrKeyringRequest req;
 	pid_t pid;
 	uid_t uid;
 	int res;
@@ -401,11 +404,12 @@ gnome_keyring_client_state_machine (GnomeKeyringClient *client)
 		client->input_pos = 0;
 		client->state = GNOME_CLIENT_STATE_READ_DISPLAYNAME;
 		break;
+		
 	case GNOME_CLIENT_STATE_READ_DISPLAYNAME:
 		debug_print (("GNOME_CLIENT_STATE_READ_DISPLAYNAME %p\n", client));
 		if (read_packet_with_size (client)) {
 			debug_print (("read packet\n"));
-			if (!gnome_keyring_proto_get_utf8_string (client->input_buffer,
+			if (!gnome_keyring_proto_get_utf8_string (&client->input_buffer,
 								  4, NULL, &str)) {
 				gnome_keyring_client_free (client);
 				return;
@@ -420,10 +424,11 @@ gnome_keyring_client_state_machine (GnomeKeyringClient *client)
 			client->state = GNOME_CLIENT_STATE_READ_PACKET;
 		}
 		break;
+		
 	case GNOME_CLIENT_STATE_READ_PACKET:
 		debug_print (("GNOME_CLIENT_STATE_READ_PACKET %p\n", client));
 		if (read_packet_with_size (client)) {
-			debug_print (("read packet, size: %d\n", client->input_buffer->len));
+			debug_print (("read packet, size: %d\n", client->input_buffer.len));
 			g_source_remove (client->input_watch);
 			client->input_watch = 0;
 			client->state = GNOME_CLIENT_STATE_COLLECT_INFO;
@@ -431,9 +436,10 @@ gnome_keyring_client_state_machine (GnomeKeyringClient *client)
 			goto new_state;
 		}
 		break;
+		
 	case GNOME_CLIENT_STATE_COLLECT_INFO:
 		debug_print (("GNOME_CLIENT_STATE_COLLECT_INFO %p\n", client));
-		if (!gnome_keyring_proto_decode_packet_operation (client->input_buffer, &op)) {
+		if (!gnome_keyring_proto_decode_packet_operation (&client->input_buffer, &op)) {
 			gnome_keyring_client_free (client);
 			return;
 		}
@@ -448,65 +454,95 @@ gnome_keyring_client_state_machine (GnomeKeyringClient *client)
 		}
 		
 		/* Make sure keyrings in memory are up to date before asking for access */
-		update_keyrings_from_disk ();
+		gkr_keyrings_update ();
 		
-		access_requests = NULL;
-		if (!keyring_ops[op].collect_info (client->input_buffer,
-						   &access_requests)) {
+		/* Setup for call */
+		memset (&req, 0, sizeof (req));
+		req.app_ref = client->app_ref;
+
+		if (!keyring_ops[op].collect_info (&client->input_buffer, &req)) {
 			gnome_keyring_client_free (client);
 			return;
 		}
+		
+		/* All the things we have to ask about */
+		client->ask_requests = req.ask_requests;
+		client->collect_data = req.data;
 
 		/* request_access can reenter here if there is no need to
 		 * wait for access rights */
 		client->state = GNOME_CLIENT_STATE_REQUEST_ACCESS;
-		ask = gnome_keyring_ask (access_requests,
-					 client->app_ref,
-					 ask_result,
-					 client);
-		if (ask != NULL) {
-			/* Don't set this if NULL, because then the client can already be destroyed
-			 * in the ask_result callback. */
-			client->ask = ask;
-		}
-		gnome_keyring_access_request_list_free (access_requests);
-		break;
+		goto new_state;
+		
 	case GNOME_CLIENT_STATE_REQUEST_ACCESS:
 		debug_print (("GNOME_CLIENT_STATE_REQUEST_ACCESS %p\n", client));
-		/* Got all data now */
-		client->state = GNOME_CLIENT_STATE_EXECUTE_OP;
-		goto new_state;
+		
+		/* Nothing should currently be asking */
+		g_assert (!client->ask);
+		
+		/* Some access requests are processed right away, so loop */
+		if (!client->ask && client->ask_requests) {
+			
+			/* Go for first item in the list */
+			client->ask = GKR_ASK_REQUEST (client->ask_requests->data);
+			client->ask_requests = g_list_remove (client->ask_requests, client->ask);
+			g_signal_connect (client->ask, "completed", G_CALLBACK (ask_result), client);
+			gkr_ask_daemon_queue (client->ask);
+			
+			/* 
+			 * If it was processed immediately, then ask_result will have 
+			 * already been called and client->ask will be NULL.
+			 */
+			return;
+		}
+		
+		/* Got all data now? */
+		if (!client->ask) {
+			g_assert (!client->ask_requests);
+			client->state = GNOME_CLIENT_STATE_EXECUTE_OP;
+			goto new_state;
+		}
 		break;
+		
 	case GNOME_CLIENT_STATE_EXECUTE_OP:
 		debug_print (("GNOME_CLIENT_STATE_EXECUTE_OP %p\n", client));
-		if (!gnome_keyring_proto_decode_packet_operation (client->input_buffer, &op)) {
+		if (!gnome_keyring_proto_decode_packet_operation (&client->input_buffer, &op)) {
 			gnome_keyring_client_free (client);
 			return;
 		}
 
-		client->output_buffer = g_string_new (NULL);
+		gkr_buffer_init_full (&client->output_buffer, 128, g_realloc);
 
 		/* Make sure keyrings in memory are up to date */
 		/* This call may remove items or keyrings, which change
 		   the client->access_requests list */
-		update_keyrings_from_disk ();
+		gkr_keyrings_update ();
 
+		/* Must have already processed all requests */
+		g_assert (!client->ask);
+		g_assert (!client->ask_requests);
 		
 		/* Add empty size */
-		gnome_keyring_proto_add_uint32 (client->output_buffer, 0);
+		gnome_keyring_proto_add_uint32 (&client->output_buffer, 0);
 		
-		if (!keyring_ops[op].execute_op (client->input_buffer,
-						 client->output_buffer,
-						 client->app_ref,
-						 client->access_requests)) {
+		/* Setup for call */
+		memset (&req, 0, sizeof (req));
+		req.app_ref = client->app_ref;
+		req.data = client->collect_data;
+		req.ask_requests = client->granted_requests;
+		
+		if (!keyring_ops[op].execute_op (&client->input_buffer,
+						 &client->output_buffer,
+						 &req)) {
 			gnome_keyring_client_free (client);
 			return;
 		}
-		gnome_keyring_access_request_list_free (client->access_requests);
-		client->access_requests = NULL;
+		
+		ask_list_free (client->granted_requests);
+		client->granted_requests = NULL;
 
-		if (!gnome_keyring_proto_set_uint32 (client->output_buffer, 0,
-						     client->output_buffer->len)) {
+		if (!gnome_keyring_proto_set_uint32 (&client->output_buffer, 0,
+						     client->output_buffer.len)) {
 			gnome_keyring_client_free (client);
 			return;
 		}
@@ -520,12 +556,13 @@ gnome_keyring_client_state_machine (GnomeKeyringClient *client)
 		client->state = GNOME_CLIENT_STATE_WRITE_REPLY;
 		goto new_state;
 		break;
+		
 	case GNOME_CLIENT_STATE_WRITE_REPLY:
 		debug_print (("GNOME_CLIENT_STATE_WRITE_REPLY %p\n", client));
-		debug_print (("writing %d bytes\n", client->output_buffer->len));
+		debug_print (("writing %d bytes\n", client->output_buffer.len));
 		res = write (client->sock,
-			     client->output_buffer->str + client->output_pos,
-			     client->output_buffer->len - client->output_pos);
+			     client->output_buffer.buf + client->output_pos,
+			     client->output_buffer.len - client->output_pos);
 		if (res <= 0) {
 			if (errno != EAGAIN &&
 			    errno != EINTR) {
@@ -535,11 +572,12 @@ gnome_keyring_client_state_machine (GnomeKeyringClient *client)
 		}
 		client->output_pos += res;
 
-		if (client->output_pos == client->output_buffer->len) {
+		if (client->output_pos == client->output_buffer.len) {
 			/* Finished operation */
 			gnome_keyring_client_free (client);
 		}
 		break;
+		
 	default:
 		break;
 	}
@@ -582,8 +620,14 @@ gnome_keyring_client_new (int fd)
 	client->state = GNOME_CLIENT_STATE_CREDENTIALS;
 	client->sock = fd;
 	client->input_channel = channel;
-	client->input_buffer = g_string_new (NULL);
 	client->input_pos = 0;
+	
+	/* 
+	 * We really have no idea what operation the client will send, 
+	 * so we err on the side of caution and use secure memory in case
+	 * passwords or secrets are involved.
+	 */  
+	gkr_buffer_init_full (&client->input_buffer, 128, gnome_keyring_memory_realloc);
 
 	clients = g_list_prepend (clients, client);
 }
@@ -626,25 +670,44 @@ new_client (GIOChannel  *channel,
 void
 cleanup_socket_dir (void)
 {
-	unlink (socket_path);
-	rmdir (tmp_dir);
+	if(*socket_path)
+		unlink (socket_path);
+	if(*tmp_dir)
+		rmdir (tmp_dir);
 }
 
 gboolean
 create_master_socket (const char **path)
 {
+	gboolean have_path = FALSE;
 	int sock;
 	struct sockaddr_un addr;
 	GIOChannel *channel;
-	gchar *tmp_tmp_dir;
+	gchar *tmp_tmp_dir = NULL;
 	
-	/* Create private directory for agent socket */
-	tmp_tmp_dir = g_build_filename (g_get_tmp_dir (), "keyring-XXXXXX", NULL);
-	strncpy (tmp_dir, tmp_tmp_dir, sizeof (tmp_dir));
-	if (mkdtemp (tmp_dir) == NULL) {
-		perror ("mkdtemp: socket dir");
-		return FALSE;
+	/* 
+	 * When run under control of unit tests, we let the parent process
+	 * pass in the socket path that we're going to create our main socket on.
+	 */
+	
+#ifdef WITH_TESTS
+	const gchar* env = g_getenv ("GNOME_KEYRING_TEST_PATH");
+	if (env && *env) {
+		g_strlcpy (tmp_dir, env, sizeof (tmp_dir));
+		have_path = TRUE;
+	} 
+#endif /* WITH_TESTS */	
+	
+	if (!have_path) {
+		/* Create private directory for agent socket */
+		tmp_tmp_dir = g_build_filename (g_get_tmp_dir (), "keyring-XXXXXX", NULL);
+		strncpy (tmp_dir, tmp_tmp_dir, sizeof (tmp_dir));
+		if (mkdtemp (tmp_dir) == NULL) {
+			perror ("mkdtemp: socket dir");
+			return FALSE;
+		}
 	}
+
 	snprintf (socket_path, sizeof (socket_path), "%s/socket", tmp_dir);
 	
 	sock = socket(AF_UNIX, SOCK_STREAM, 0);
