@@ -36,22 +36,26 @@ typedef enum {
 	GNOME_KEYRING_RESULT_BAD_ARGUMENTS,
 	GNOME_KEYRING_RESULT_IO_ERROR,
 	GNOME_KEYRING_RESULT_CANCELLED,
-	GNOME_KEYRING_RESULT_ALREADY_EXISTS
+	GNOME_KEYRING_RESULT_KEYRING_ALREADY_EXISTS
 } GnomeKeyringResult;
 
+#define GNOME_KEYRING_RESULT_ALREADY_EXISTS \
+	GNOME_KEYRING_RESULT_KEYRING_ALREADY_EXISTS
+
 typedef enum {
-	/* The item types */
-	GNOME_KEYRING_ITEM_GENERIC_SECRET,
-	GNOME_KEYRING_ITEM_NETWORK_PASSWORD,
-	GNOME_KEYRING_ITEM_NOTE,
-	
-	/* Used internally */
-	GNOME_KEYRING_ITEM_LAST_TYPE,
 	GNOME_KEYRING_ITEM_TYPE_MASK = 0x0000ffff,
 	GNOME_KEYRING_ITEM_NO_TYPE = GNOME_KEYRING_ITEM_TYPE_MASK,
 
-	/* Makes a item only for applications in ACL */
-	GNOME_KEYRING_ITEM_APPLICATION_SECRET = 0x01000000,
+	/* The item types */
+	GNOME_KEYRING_ITEM_GENERIC_SECRET = 0,
+	GNOME_KEYRING_ITEM_NETWORK_PASSWORD,
+	GNOME_KEYRING_ITEM_NOTE,
+	
+	/*< private >*/
+	GNOME_KEYRING_ITEM_LAST_TYPE,
+
+	/* Makes a item only for application that created it */
+	GNOME_KEYRING_ITEM_APPLICATION_SECRET = 0x01000000
 	
 } GnomeKeyringItemType;
 
@@ -103,6 +107,8 @@ typedef struct {
 	char *secret;
 } GnomeKeyringFound;
 
+void gnome_keyring_string_list_free (GList *strings);
+
 typedef void (*GnomeKeyringOperationDoneCallback)           (GnomeKeyringResult result,
 							     gpointer           data);
 typedef void (*GnomeKeyringOperationGetStringCallback)      (GnomeKeyringResult result,
@@ -127,18 +133,20 @@ typedef void (*GnomeKeyringOperationGetAttributesCallback)  (GnomeKeyringResult
 #define gnome_keyring_attribute_list_index(a, i) g_array_index ((a), GnomeKeyringAttribute, (i))
 #define gnome_keyring_attribute_list_new() (g_array_new (FALSE, FALSE, sizeof (GnomeKeyringAttribute)))
 void                       gnome_keyring_attribute_list_append_string (GnomeKeyringAttributeList *attributes,
-								       const char                *attribute,
+								       const char                *name,
 								       const char                *value);
 void                       gnome_keyring_attribute_list_append_uint32 (GnomeKeyringAttributeList *attributes,
-								       const char                *attribute,
+								       const char                *name,
 								       guint32                    value);
 void                       gnome_keyring_attribute_list_free          (GnomeKeyringAttributeList *attributes);
 GnomeKeyringAttributeList *gnome_keyring_attribute_list_copy          (GnomeKeyringAttributeList *attributes);
 
 
+const gchar*               gnome_keyring_result_to_message            (GnomeKeyringResult res);
+
 gboolean gnome_keyring_is_available (void);
 
-void gnome_keyring_free_password (char *password);
+void gnome_keyring_free_password (gchar *password);
 
 void gnome_keyring_found_free (GnomeKeyringFound *found);
 void gnome_keyring_found_list_free (GList *found_list);
@@ -336,16 +344,15 @@ GnomeKeyringResult gnome_keyring_item_set_acl_sync        (const char
 							   guint32                                     id,
 							   GList                                      *acl);
 
-/*
- * Will grant the application access rights to the item, provided 
- * callee has, or is granted, write access to said item.
- *
- * keyring      : Name of keyring to access or NULL for default keyring
- * display_name : Display name of application to be granted access rights - As returned by g_get_application_name()
- * full_path    : Full path of application to be granted access rights
- * id           : ID of item to access in keyring
- * rights       : Bitmask of rights to grant the application 
- */
+gpointer           gnome_keyring_item_grant_access_rights      (const gchar                       *keyring, 
+                                                                const gchar                       *display_name, 
+                                                                const gchar                       *full_path, 
+                                                                const guint32                      id, 
+                                                                const GnomeKeyringAccessType       rights,
+                                                                GnomeKeyringOperationDoneCallback  callback,
+                                                                gpointer                           data,
+                                                                GDestroyNotify                     destroy_data);
+
 GnomeKeyringResult gnome_keyring_item_grant_access_rights_sync (const char                   *keyring, 
 								const char                   *display_name,  
 								const char                   *full_path,  
